@@ -1,4 +1,6 @@
-package C::Blocks;
+########################################################################
+                       package C::Blocks;
+########################################################################
 
 use strict;
 use warnings;
@@ -9,14 +11,14 @@ use XSLoader;
 # Use David Golden's version numbering suggestions. Note that we have to call
 # the XSLoader before evaling the version string because XS modules check the
 # version *string*, not the version *number*, at boot time.
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 XSLoader::load('C::Blocks', $VERSION);
 $VERSION = eval $VERSION;
 
-our (@__code_cache_array, @__symtab_cache_array);
+our (@__code_cache_array, @__symtab_cache_array, @__dll_list_array);
 our $default_compiler_options = "-Wall -D_C_BLOCKS_OS_$^O ";
 our $compiler_options = $default_compiler_options;
-our $library_to_link;
+our @libraries_to_link;
 our ($_add_msg_functions, $_msg);
 
 sub import {
@@ -27,6 +29,7 @@ sub import {
 	*{$caller.'::csub'} = sub () {};
 	*{$caller.'::cshare'} = sub () {};
 	*{$caller.'::clex'} = sub () {};
+	*{$caller.'::cisa'} = sub () {};
 	_import();
 }
 
@@ -55,6 +58,70 @@ END {
 	_cleanup();
 }
 
+########################################################################
+                   package C::Blocks::Type::NV;
+########################################################################
+use Scalar::Util;
+use Carp;
+
+our $TYPE = 'NV';
+our $INIT = 'SvNV';
+our $CLEANUP = 'sv_setnv';
+
+sub check_var_types {
+	my $package = shift @_;
+	$@ = '';
+	while (@_) {
+		my ($arg_name, $arg) = splice @_, 0, 2;
+		$@ .= "$arg_name is not defined\n" and next if not defined $arg;
+		$@ .= "$arg_name is a reference\n" and next if ref($arg);
+		$@ .= "$arg_name does not look like a number" and next
+			unless Scalar::Util::looks_like_number($arg);
+	}
+	if ($@ eq '') {
+		undef $@;
+		return 1;
+	}
+	return 0;
+}
+
+########################################################################
+               package C::Blocks::Type::double;
+########################################################################
+our $TYPE = 'double';
+our $INIT = 'SvNV';
+our $CLEANUP = 'sv_setnv';
+*check_var_types = \&C::Blocks::Type::NV::check_var_types;
+
+########################################################################
+               package C::Blocks::Type::float;
+########################################################################
+our $TYPE = 'float';
+our $INIT = 'SvNV';
+our $CLEANUP = 'sv_setnv';
+*check_var_types = \&C::Blocks::Type::NV::check_var_types;
+
+########################################################################
+               package C::Blocks::Type::int;
+########################################################################
+our $TYPE = 'int';
+our $INIT = 'SvIV';
+our $CLEANUP = 'sv_setiv';
+*check_var_types = \&C::Blocks::Type::NV::check_var_types;
+
+########################################################################
+               package C::Blocks::Type::uint;
+########################################################################
+our $TYPE = 'unsigned int';
+our $INIT = 'SvUV';
+our $CLEANUP = 'sv_setuv';
+# Should check sign, too
+*check_var_types = \&C::Blocks::Type::NV::check_var_types;
+
+# Other types:
+# int2ptr
+# uint2ptr
+
 1;
 
 __END__
@@ -95,7 +162,31 @@ C::Blocks - embeding a fast C compiler directly into your Perl parser
      print_location(2);
  }
  
- # NOTE: csub does not yet work, for some odd reason
+ # A function that sums all of the arguments
+ csub csum {
+     /* get "items" variable, and stack pointer
+      * variables used by ST() */
+     dXSARGS;
+ 
+     int i;
+     double sum = 0.;
+     
+     /* Sum the given numeric values. */
+     for (i = 0; i < items; ++i) sum += SvNV( ST(i) );
+     
+     /* Prepare stack to receive return values. */
+     XSprePUSH;
+     /* Push the sum onto the return stack */
+     mXPUSHn(sum);
+     /* Indicate we're returning a single value
+      * on the stack. */
+     XSRETURN(1);
+ }
+ 
+ my $limit = shift || 5;
+ my $return = csum(1 .. $limit);
+ print "sum of 1 to $limit is $return\n";
+ 
  
  ### In file My/Fastlib.pm
  
@@ -120,6 +211,18 @@ C::Blocks - embeding a fast C compiler directly into your Perl parser
      say_hi();
  }
  
+ ### Use Perl to generate code at compile time
+ 
+ # Create a preprocessor string with the full
+ # path to our configuration file
+ use File::HomeDir;
+ use File::Spec;
+ clex {
+     #define CONF_FILE_NAME ${ '"' .
+         File::Spec->catfile(File::HomeDir->my_home, 'myconf.txt')
+          . '"' }
+ }
+ 
  print "All done!\n"; 
 
 =head1 ALPHA
@@ -374,6 +477,61 @@ import behavior. You fix that by providing your own C<import> method:
      goto &$method;
  }
 
+=head2 XSUBs
+
+Since the advent of Perl 5, the XS toolchain has made it possible to
+write C functions that can be called directly by Perl code. Such
+functions are referred to as XSUBs. C::Blocks provides similar
+functionality via the C<csub> keyword. Just like Perl's C<sub> keyword,
+this keyword takes the name of the function and a code block, and
+produces a function in the current package with the given name.
+
+Writing a functional XSUB requires knowing a fair bit about the Perl
+argument stack and manipulation macros, so it will have to be discussed
+at greater depth somewhere else. For now, it may be best to refer to
+L<perlapi> and L<http://blog.booking.com/native-extensions-for-perl-without-smoke-and-mirrors.html>.
+
+=head2 Generating C Code
+
+Because C<C::Blocks> hooks on keywords, it is naturally invoked in
+C<cblock>, C<cshare>, C<clex>, and C<csub> blocks which are themselves
+contained within a string eval. However, string evals compile at
+runtime, not script parse time. Although it would be easy to generate C
+code using Perl, writing useful C<clex> and C<cshare> blocks is tricky.
+
+For this reason, C<C::Blocks> provides a bit of notation for an
+"interpolation block." An interpolation block is a block of Perl code
+that is run as soon as it is extracted (i.e. during script compile time).
+The return value is then inserted directly into the text that gets
+compiled. Thus, these two C<cblock>s end up doing the same thing:
+
+ cblock {
+     printf("Hello!\n");
+ }
+ 
+ cblock {
+     ${ 'printf' } ("Hello!\n");
+ }
+
+The example given in the L</SYNOPSIS> is probably more meaningful. It
+also illustrates that the value returned by the Perl code has to be
+literal C code, including the left and right double quotes for strings.
+This arises because sigils (and interpolation blocks by extension, as
+they are delimited by a sigil) are ignored within strings and C comments.
+
+Note: The current implementation is unpolished. In particular, it does
+not intelligently handle exceptions thrown during the evaluation of the
+Perl code. (Indeed, at the moment it suppresses them.)
+
+For the most part, any side effects from the code contained in
+interpolation blocks behave exactly like side effects from BEGIN blocks.
+There is an exception, however, for Perls earlier than 5.18. In these
+older Perls, lexical variables become uninitialized after all interpolation
+blocks execute, but before any BEGIN blocks run. This only applies to
+lexically scoped variables, however. Changes to package-scoped variables
+(including lexically scoped names, i.e. C<our $package_var>) persist,
+as would be expected if these variables were set in BEGIN blocks.
+
 =head2 Performance
 
 C<C::Blocks> is currently implemented using the Tiny C Compiler, a 
@@ -398,6 +556,40 @@ you declare your C data structures? And more importantly, how do you
 package those structures into a library in order to share those 
 structures with others? That's what I discuss next.
 
+=head2 Configuring the Compiler
+
+Sometimes you need to configure the compiler. The most common situation
+that arises involves linking against external libraries. You may also
+need to include traditional compiler command-line arguments which you
+obtain from the command-line, or from a configuration module. The
+current means to do this is by setting special-purposed package
+variables which get examined during the compilation stage.
+
+NOTE: THIS API IS UNDER DEVELOPMENT. UNTIL C::BLOCKS REACHES v1.0, THIS
+API IS SUBJECT TO CHANGE, LIKELY WITHOUT NOTICE.
+
+To set most compiler settings, you simply treat
+C<$C::Blocks::compiler_options> like the command-line. For example, if
+you want to set preprocessor definition at runtime (but don't want to
+use an interpolation block for some reason), you can use
+
+ BEGIN { $C::Blocks::compiler_options = '-DDEBUG' }
+
+A very important aspect to remember is that the compiler options, and
+the shared libraries mentioned next, only apply to the first block they
+encounter. The process of compiling a block clears these variables.
+
+For C::Blocks, tcc does not handle linking to shared libraries, because
+it does not know how to open shared libraries on Mac systems. Instead,
+C::Blocks manages the shared libraries itself, loading libraries and
+looking up symbols using Dynaloader. For this reason, the shared
+libraries are not indicated with the typical C<-L> and C<-l> flags as
+compiler options. Instead, each library should be added to the package
+variable C<@C::Blocks::libraries_to_link>. Each string in this list
+should be the full library name, including file extensions. If the
+library is located in an unconventional location, the full path should
+be specified.
+
 =head1 KEYWORDS
 
 The way that C<C::Blocks> provides these functionalities is through lexically
@@ -406,6 +598,10 @@ precede a block of C code encapsulated in curly brackets. Because these use the
 Perl keyword API, they parse the C code during Perl's parse stage, so any code
 errors in your C code will be caught during parse time, not during run time.
 
+In addition to these keywords, C<C::Blocks> lets you indicate types and
+type conversion with C<cisa>. Unlike the other keywords, this keyword is
+not followed by a block of code, but the type and a list of variables.
+
 =over
 
 =item cblock { code }
@@ -421,10 +617,13 @@ definitions are confined to the C<cblock> and will not be present in later
 C<cblock>s. For that sort of behavior, see C<clex>.
 
 Variables with C<$> sigils are interpreted as referring to the C<SV*>
-representing the variable in the current lexical scope.
+representing the variable in the current lexical scope, unless otherwise
+specified with a C<cisa> statement.
 
 Note: If you need to leave a C<cblock> early, you should use a C<return>
-statement without any arguments.
+statement without any arguments. This will also bypass the data repacking
+provided by C<cisa> types.
+
 
 =item clex { code }
 
@@ -450,6 +649,37 @@ other xsub.
 
 Currently, C<csub> does not work.
 
+=item cisa type variable-list
+
+If you include sigil variables in your C<cblock> blocks (not C<clex>,
+C<cshare>, or C<csub>, just C<cblock>), they will normally be resolved
+to the underlying SV data structure for that variable. Under many
+circumstances, you do not need to manipulate the SV itself, but merely
+need the data contained in the SV (or the object pointed to by the SV).
+A C<cisa> statement tells C::Blocks that certain variables should be
+represented by a C data structure other than an SV. The package used for
+the type (must) have package constants that indicate the C type to use,
+and how to marshall the data at the beginning and end of your block.
+
+C<cisa> statements also have the runtime responsibility of validating
+the data in the variables. Failed validations should probably throw
+exceptions indicating which variables did not satisfy validation, and
+why they failed. Your validation code can make as much or as little
+noise as you deem appropriate, from quietly setting C<$@> to warning to
+throwing exceptions. Note that you could include validation code in the
+initialization function, but C<cisa> validation is only called once per
+C<cisa> statement, whereas the variable initialization code is called
+at the beginning of each C<cblock> that uses the variable.
+
+Packages that represent types must include the package variables
+C<$TYPE> and C<$INIT>. The first indicates the C type while the second
+indicates a C macro or function that accepts an SV and returns the
+data of type C<$TYPE>. C<$CLEANUP> is an optional macro or function that
+takes the original SV and the (presumably revised) data, and updates the
+contents of the SV. Runtime type checking is performed by the package
+method C<check_var_types>, which gets key/value pairs of the variable
+name and the variable.
+
 =back
 
 =head1 SEE ALSO
